format zx81
;labelusenumeric
;LISTOFF
	MEMAVL	   EQU	   MEM_32K
	STARTMODE  EQU	   SLOW_MODE
	DFILETYPE  EQU	   COLLAPSED
	include '..\..\SINCL-ZX\ZX81.INC'
;LISTON

//--------------------------------
	REM _asm
// Here are the things to change depending on the video
FileName	dbzx	'BBB25FS.RVC'
		db	0 //WARNING: do not alter or remove this line
NR_OF_FRAMES	=	1091
IMAGE_HEIGHT	=	24 //widescreen(16:9)=18, fullscreen(4:3)=24
TOTAL_RASTERS	=	312 //50Hz(PAL)=312, 60Hz(NTSC)=262, 48Hz(Film)=325

//--------------------------------
CHAR_SET	=	8 * 1024 //compatible with Chroma and UDG4ZXpand
IMAGE_0 	=	32 * 1024 //$8000
IMAGE_1 	=	33 * 1024 //$8400

Main:
	ld	ix,DRAW_IMAGE

	StartPlay:
		//open video file
		ld	de,FileName
		call	OpenFile

		//read the fixed greyscale blocks
		if IMAGE_HEIGHT = 24
			call	SyncAndRequest256BytesFromFile
			ld	hl,CHAR_SET + 768
			call	Read256BytesFromFileToHL
		else if IMAGE_HEIGHT = 18
			//init static characters
			call	SyncAndRequest256BytesFromFile
			ld	hl,CHAR_SET + 256 - 64
			call	ReadFirst64BytesFromFileToHL
			ld	hl,CHAR_SET + 512 - 64
			call	Read64BytesFromFileToHL
			ld	hl,CHAR_SET + 768 - 64
			call	Read64BytesFromFileToHL
			ld	hl,CHAR_SET + 1024 - 64
			call	Read64BytesFromFileToHL
		end if

		//init character set pointers
		ld	a,CHAR_SET / 256
		ld	(CharSetPtr),a
		inc	a  //for 128 characters mode (EightyOne only?)
		ld	i,a

		//nr of frames in video
		ld	bc,NR_OF_FRAMES

		PlayLoop:
			push	bc

			if IMAGE_HEIGHT = 24
				//read image data, first part
				call	Request256BytesFromFile
				ImageStartPtr = $ + 1
				ld	hl,IMAGE_0
				call	Read256BytesFromFileToHL

				//read image data, second part
				if TOTAL_RASTERS = 262
					push	hl
					call	SyncAndRequest256BytesFromFile
					pop	hl
				else
					call	Request256BytesFromFile
				end if
				call	Read256BytesFromFileToHL

				//read image data, third part
				push	hl
				call	Request256BytesFromFile
				pop	hl
				call	Read256BytesFromFileToHL

				//read character set data
				push	hl
				call	SyncAndRequest256BytesFromFile
				pop	hl
				CharSetPtr = $ + 2
				ld	hl,CHAR_SET
				call	Read256BytesFromFileToHL
			else if IMAGE_HEIGHT = 18
				//read image data, first part
				call	SyncAndRequest256BytesFromFile
				ImageStartPtr = $ + 1
				ld	hl,IMAGE_0
				call	Read256BytesFromFileToHL

				//read image data, second part
				call	Request256BytesFromFile
				call	Read256BytesFromFileToHL

				//read image data, third part
				push	hl
				call	SyncAndRequest256BytesFromFile
				pop	hl
				call	ReadFirst64BytesFromFileToHL
				//read character set data
				CharSetPtr = $ + 2
				ld	hl,CHAR_SET
				call	Read192BytesFromFileToHL
			end if

			//swap image shown
			ld	hl,(ImageStartPtr)
			ld	(ImageStart),hl
			//change where to write the next image
			ld	a,h
			xor	4 //(IMAGE_0 xor IMAGE_1) >> 8
			ld	(ImageStartPtr + 1),a
			//change where to write in the character set next time
			ld	a,(CharSetPtr)
			inc	a
			if IMAGE_HEIGHT = 24
				cp	(CHAR_SET / 256) + 3
			else if IMAGE_HEIGHT = 18
				cp	(CHAR_SET / 256) + 4
			end if
			jr	nz,.NoReset
			ld	a,(CHAR_SET / 256) + 0
		.NoReset:
			ld	(CharSetPtr),a

			//count nr of frames shown, loop
			pop	bc
			dec	bc
			ld	a,b
			or	c
		jp	nz,PlayLoop

	call	CloseFile
	jp	StartPlay

//--------------------------------
PRT0	EQU	$0007
PRT1	EQU	$2007
PRT2	EQU	$4007
PRT3	EQU	$6007
PRT4	EQU	$8007
PRT5	EQU	$a007
PRT6	EQU	$c007
PRT7	EQU	$e007

IMAGE_WIDTH	= 32
IMAGE_RASTERS	= IMAGE_HEIGHT * 8
VSYNC_RASTERS	= 4
WASTED_RASTERS	= 4
TOTAL_MARGIN	= TOTAL_RASTERS - IMAGE_RASTERS - VSYNC_RASTERS - WASTED_RASTERS
TOP_MARGIN	= TOTAL_MARGIN / 2
BOTTOM_MARGIN	= TOTAL_MARGIN - TOP_MARGIN

StackSave	dw	0
FrameCounter	db	0

//--------------------------------
SyncAndRequest256BytesFromFile:
//request 256 bytes from file
	ld	bc,PRT5
	ld	a,0
	out	(c),a
//sync with DEAFULT_IMAGE
	call	WaitScreen
//wait for zxpand to finish command
//_should_ be done directly
	.DelayStart:
		in	a,(c)
	jp	m,.DelayStart
	ret

//--------------------------------
Request256BytesFromFile:
//request 256 bytes from file
	ld	bc,PRT5
	ld	a,0
	out	(c),a
//wait for zxpand to finish command
	.DelayStart:
		nop
		in	a,(c)
	jp	m,.DelayStart
	ret

//--------------------------------
Read256BytesFromFileToHL:
//prepare zxpand buffer for read
	ld	bc,PRT0
	ld	a,0
	out	(c),a
//a short delay
	repeat 6
		nop
	end repeat
//fast read 256 bytes from zxpand to memory
	repeat 256
		ini
	end repeat
	ret

//--------------------------------
ReadFirst64BytesFromFileToHL:
//prepare zxpand buffer for read
	ld	bc,PRT0
	ld	a,0
	out	(c),a
//a short delay
	repeat 6
		nop
	end repeat
Read64BytesFromFileToHL:
//fast read 64 bytes from zxpand to memory
	repeat 64
		ini
	end repeat
	ret

//--------------------------------
Read192BytesFromFileToHL:
//fast read 192 bytes from zxpand to memory
	repeat 192
		ini
	end repeat
	ret

//--------------------------------
//de = file path, zero-terminated zx-string
OpenFile:
//prepare zxpand buffer for write
	ld	bc,PRT0
	ld	a,$ff
	out	(c),a
//send filename to zxpand
	ld	bc,PRT2
	.NameOutLoop:
		ld	a,(de)
		out	(c),a		//one character to zxpand
		inc	de
		and	a
	jp	nz,.NameOutLoop 	//repeat until zero-termination
//open file for read
	ld	bc,PRT4
	xor	a
	out	(c),a
	call	GetResponse
//do not read meta-data, not needed
	ret

//--------------------------------
CloseFile:
	ld	bc,PRT4
	ld	a,$80
	out	(c),a
	call	GetResponse
	ret

//--------------------------------
GetResponse:
	.DelayStart:
		in	a,(c)		//get response code
	jp	m,.DelayStart		//and loop while negative
	cp	$40			//all ok
	ret	z			//ret if ok
ErrorHandling:
	ld	a,$1f
	ld	i,a
	ld	hl,DEAFULT_IMAGE
	ld	(ImageStart),hl
	jp	$			//else stop

//--------------------------------
//sync to directly after image update
WaitScreen:
	ld	hl,FrameCounter
	ld	a,(hl)
	.FrameLoop:
		cp	(hl)
	jp	z,.FrameLoop
	ret

//--------------------------------
DRAW_IMAGE:
//save shadow registers
	exx				//4
	push	bc			//11
	push	de			//11
	push	hl			//11
//init shadow registers
	ld	hl,(ImageStart) 	//16
	ld	de,IMAGE_WIDTH		//10
	add	hl,de			//11
	ld	c,$c9			//7 $c9 = ret
//change dfile+32 to a ret
	ld	b,(hl)			//7
	ld	(hl),c			//7
	exx				//4

//save real stack
	ld	(StackSave),sp		//20
//init line-stack
	ld	sp,LINE_STACK_START	//10
	ImageStart = $ + 1
	ld	hl,DEAFULT_IMAGE	//10
	set	7,h			//8
	ld	de,IMAGE_WIDTH		//10
//waste time
	repeat 42
		nop			//4 * repeat
	end repeat
//reset LINECNTR in ULA
	in	a,($fe) 		//11
	out	($ff),a 		//11
//waste time
	dec	bc			//6
	nop				//4
	nop				//4
	nop				//4
	nop				//4
//nr of rasters to draw
	ld	b,IMAGE_RASTERS 	//7
//draw the picture as specified
	jp	(hl)			//4

//--------------------------------
AFTER_LINE0:
//dec line counter
	dec	b			//4
	jp	z,DRAW_IMAGE_DONE	//10/10 previous line was last
//reinit line stack
	ld	sp,LINE_STACK		//10
//waste time
	repeat 9
		nop			//4 * 9 = 36
	end repeat
	ret	z			//5 //65
//draw 1 raster of 32 chars
	jp	(hl)			//4 + (32 * 4) + 10 = 142 (65 left)

//--------------------------------
AFTER_LINE1_6:
//dec line counter
	dec	b			//4
	jp	z,DRAW_IMAGE_DONE	//10/10 previous line was last
//waste time
	repeat 11
		nop			//4 * 11 = 44
	end repeat
	xor	0			//7 //65
//draw 1 raster of 32 chars
	jp	(hl)			//4 + (32 * 4) + 10 = 142 (65 left)

//--------------------------------
AFTER_LINE7:
//dec line counter
	dec	b			//4
	jp	z,DRAW_IMAGE_DONE	//10/10 previous line was last
//point to next line of text
	add	hl,de			//11
//restore old dfile-data
	exx				//4
	ld	(hl),b			//7
//change dfile+32 to a ret
	add	hl,de			//11
	ld	b,(hl)			//7
	ld	(hl),c			//7
	exx				//4 //65
//draw 1 raster of 32 chars
	jp	(hl)			//4 + (32 * 4) + 10 = 142 (65 left)

//--------------------------------
DRAW_IMAGE_DONE:
//restore stack
	ld	sp,(StackSave)		//20
//restore dfile+32
	exx				//4
	ld	(hl),b			//7
	exx				//4
//increment FrameCounter
	ld	hl,FrameCounter 	//10
	inc	(hl)			//11
//prepare for bottom margin and VSYNC
	BottomMargin = $ + 1
	ld	a,BOTTOM_MARGIN 	//7
	neg				//8
	inc	a			//4
	ex	af,af'			//4
	ld	ix,GENERATE_VSYNC	//14
//restore shadow registers
	exx
	pop	hl			//10
	pop	de			//10
	pop	bc			//10
	exx
//restore registers
	pop	hl			//10
	pop	de			//10
	pop	bc			//10
	pop	af			//10
//NMI on
	out	($fe),a 		//11
//return to application
	ret				//10

//--------------------------------
GENERATE_VSYNC:
//VSync start
	in	a,($fe) 		//11
//waste time, 4 rasters worth of VSync
	ld	b,57			//7
	djnz	$			//13/8
//prepare for top margin
	TopMargin = $ + 1
	ld	a,TOP_MARGIN		//7
	neg				//8
	inc	a			//4
//prepare a' for NMI-counter
	ex	af,af'			//4
	ld	ix,DRAW_IMAGE		//14
//restore registers
	pop	hl			//10
	pop	de			//10
	pop	bc			//10
	pop	af			//10
//NMI on, VSync stop
	out	($fe),a 		//11
//return to application
	ret				//10

//--------------------------------
//It would have been more logical to have a stack 0-7, but there are
//no cycles left after line7.
//The solution is to rotate the line-stack so that line0 is the last,
//and deal with the arithmetic that entails.
align 256
LINE_STACK:
	dw	AFTER_LINE1_6
	dw	AFTER_LINE1_6
	dw	AFTER_LINE1_6
	dw	AFTER_LINE1_6
	dw	AFTER_LINE1_6
	dw	AFTER_LINE1_6
	dw	AFTER_LINE7
LINE_STACK_START:
	dw	AFTER_LINE0

//--------------------------------
END	_asm

//--------------------------------
AUTORUN:
	RAND USR #Main

include '..\..\SINCL-ZX\ZX81DISP.INC'	//include D_FILE and needed memory areas

//--------------------------------
DEAFULT_IMAGE:
	repeat IMAGE_WIDTH * IMAGE_HEIGHT / 16
		dbzx	'RVC-PLAYER  v0.4'
	end repeat

//--------------------------------
VARS_ADDR:
	db 80h				//DO NOT REMOVE!!!
WORKSPACE:
assert ($-MEMST)<MEMAVL
// end of program
